# -*- coding: utf-8 -*-

"""
our models
"""

from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.db import models

from allianceauth.services.modules.discord.models import DiscordUser


class AaDiscordPingFormatter(models.Model):
    """Meta model for app permissions"""

    class Meta:
        """AaDiscordPingFormatter :: Meta"""

        managed = False
        default_permissions = ()
        permissions = (("basic_access", "Can access this app"),)


# FleetComm Model
class FleetComm(models.Model):
    """Fleet Comms"""

    name = models.CharField(
        max_length=255, unique=True, help_text="Short name to identify this comms"
    )
    notes = models.TextField(
        null=True,
        default=None,
        blank=True,
        help_text="You can add notes about this configuration here if you want",
    )
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text="Whether this comms is enabled or not",
    )

    def __str__(self) -> str:
        return str(self.name)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(id={self.id}, name='{self.name}')"

    class Meta:
        """FleetComm :: Meta"""

        verbose_name = "Fleet Comm"
        verbose_name_plural = "Fleet Comms"
        default_permissions = ()


# FleetDoctrine Model
class FleetDoctrine(models.Model):
    """Fleet Doctrine"""

    name = models.CharField(
        max_length=255, unique=True, help_text="Short name to identify this doctrine"
    )
    notes = models.TextField(
        null=True,
        default=None,
        blank=True,
        help_text="You can add notes about this configuration here if you want",
    )
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text="Whether this doctrine is enabled or not",
    )

    def __str__(self) -> str:
        return str(self.name)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(id={self.id}, name='{self.name}')"

    class Meta:
        """FleetDoctrine :: Meta"""

        verbose_name = "Fleet Doctrine"
        verbose_name_plural = "Fleet Doctrines"
        default_permissions = ()


# FormupLocation Model
class FormupLocation(models.Model):
    """Formup Location"""

    name = models.CharField(
        max_length=255,
        unique=True,
        help_text="Short name to identify this formup location",
    )
    notes = models.TextField(
        null=True,
        default=None,
        blank=True,
        help_text="You can add notes about this configuration here if you want",
    )
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text="Whether this formup location is enabled or not",
    )

    def __str__(self) -> str:
        return str(self.name)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(id={self.id}, name='{self.name}')"

    class Meta:
        """FormupLocation :: Meta"""

        verbose_name = "Formup Location"
        verbose_name_plural = "Formup Locations"
        default_permissions = ()


# DiscordPingTargets Model
class DiscordPingTargets(models.Model):
    """Discord Ping Targets"""

    name = models.OneToOneField(
        Group,
        on_delete=models.CASCADE,
        unique=True,
        help_text="Name of the Discord role to ping",
    )
    discord_id = models.CharField(
        max_length=255,
        unique=True,
        blank=True,
        help_text="ID of the Discord role to ping",
    )
    restricted_to_group = models.ManyToManyField(
        Group,
        related_name="discord_role_require_groups",
        help_text=("Restrict Discord group to the following group(s) ..."),
    )
    notes = models.TextField(
        null=True,
        default=None,
        blank=True,
        help_text="You can add notes about this configuration here if you want",
    )
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text="Whether this formup location is enabled or not",
    )

    def clean(self, *args, **kwargs):
        """
        check if the group has already been synched to Discord
        :param args:
        :param kwargs:
        """
        discord_group_info = DiscordUser.objects.group_to_role(self.name)

        if not discord_group_info:
            raise ValidationError("This group has not been synched to Discord yet.")

        super().clean(*args, **kwargs)

    def save(self, *args, **kwargs):
        """
        Save everything
        :param args:
        :param kwargs:
        """
        discord_group_info = DiscordUser.objects.group_to_role(self.name)
        self.discord_id = discord_group_info["id"]
        super().save(*args, **kwargs)  # Call the "real" save() method.

    def __str__(self) -> str:
        return str(self.name)

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}("
            f"id={self.id}, "
            f"discord_id='{self.discord_id}', "
            f"restricted_to_group='{self.restricted_to_group.all()}', "
            f"name='{self.name}'"
            f") "
        )

    class Meta:
        """DiscordPingTargets :: Meta"""

        verbose_name = "Discord Ping Target"
        verbose_name_plural = "Discord Ping Targets"
        default_permissions = ()


# FleetType Model
class FleetType(models.Model):
    """Fleet Types"""

    name = models.CharField(
        max_length=255,
        unique=True,
        help_text="Short name to identify this fleet type",
    )
    embed_color = models.CharField(
        max_length=7,
        blank=True,
        help_text="Hightlight color for the embed",
    )
    notes = models.TextField(
        null=True,
        default=None,
        blank=True,
        help_text="You can add notes about this configuration here if you want",
    )
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text="Whether this fleet type is enabled or not",
    )

    def __str__(self) -> str:
        return str(self.name)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(id={self.id}, name='{self.name}')"

    class Meta:
        """FleetType :: Meta"""

        verbose_name = "Fleet Type"
        verbose_name_plural = "Fleet Types"
        default_permissions = ()


# Webhook Model
class Webhook(models.Model):
    """A Discord or Slack webhook"""

    WEBHOOK_TYPE_DISCORD = "Discord"
    WEBHOOK_TYPE_SLACK = "Slack"
    WEBHOOK_TYPE_CHOICES = (
        (WEBHOOK_TYPE_DISCORD, "Discord"),
        (WEBHOOK_TYPE_SLACK, "Slack"),
    )

    type = models.CharField(
        max_length=7,
        choices=WEBHOOK_TYPE_CHOICES,
        default=WEBHOOK_TYPE_DISCORD,
        help_text="Is this a Discord or Slack webhook?",
    )
    name = models.CharField(
        max_length=255,
        unique=True,
        help_text="Name of the channel this webhook posts to",
    )
    url = models.CharField(
        max_length=255,
        unique=True,
        help_text=(
            "URL of this webhook, e.g. "
            "https://discordapp.com/api/webhooks/123456/abcdef "
            "or https://hooks.slack.com/services/xxxx/xxxx"
        ),
    )
    is_embedded = models.BooleanField(
        default=True,
        db_index=True,
        help_text="Whether this webhook's ping is embedded or not",
    )
    restricted_to_group = models.ManyToManyField(
        Group,
        related_name="webhook_require_groups",
        help_text=("Restrict this webhook to the following group(s) ..."),
    )
    notes = models.TextField(
        null=True,
        default=None,
        blank=True,
        help_text="You can add notes about this webhook here if you want",
    )
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text="Whether this webhook is active or not",
    )

    def __str__(self) -> str:
        return str(self.name)

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}("
            f"id={self.id}, "
            f"type='{self.type}', "
            f"url='{self.url}', "
            f"restricted_to_group='{self.restricted_to_group.all()}', "
            f"name='{self.name}', "
            f"is_embedded='{self.is_embedded}'"
            f")"
        )

    class Meta:
        """Webhook :: Meta"""

        verbose_name = "Webhook"
        verbose_name_plural = "Webhooks"
        default_permissions = ()



# -*- coding: utf-8 -*-

"""
the views
"""

from discordpingformatter.models import (
    FleetComm,
    DiscordPingTargets,
    FleetType,
    Webhook,
    FleetDoctrine,
    FormupLocation,
)

from django.shortcuts import render
from django.contrib.auth.decorators import login_required, permission_required

from . import __title__

from .app_settings import (
    AA_FLEETPINGFORMATTER_USE_SLACK,
    get_site_url,
    timezones_installed,
)


@login_required
@permission_required("discordpingformatter.basic_access")
def index(request):
    """
    Index view
    """
    fleet_comms = FleetComm.objects.filter(is_enabled=True).order_by("name")

    discord_ping_targets = {}
    if AA_FLEETPINGFORMATTER_USE_SLACK is False:
        discord_ping_targets = (
            DiscordPingTargets.objects.filter(
                restricted_to_group__in=request.user.groups.all(), is_enabled=True
            )
            .distinct()
            .order_by("name")
        )

    fleet_types = FleetType.objects.filter(is_enabled=True).order_by("name")

    if AA_FLEETPINGFORMATTER_USE_SLACK is True:
        webhooks = (
            Webhook.objects.filter(
                type="Slack",
                restricted_to_group__in=request.user.groups.all(),
                is_enabled=True,
            )
            .distinct()
            .order_by("name")
        )
    else:
        webhooks = (
            Webhook.objects.filter(
                type="Discord",
                restricted_to_group__in=request.user.groups.all(),
                is_enabled=True,
            )
            .distinct()
            .order_by("name")
        )

    doctrines = FleetDoctrine.objects.filter(is_enabled=True).order_by("name")
    formup_locations = FormupLocation.objects.filter(is_enabled=True).order_by("name")

    context = {
        "title": __title__,
        "additionalPingTargets": discord_ping_targets,
        "additionalFleetTypes": fleet_types,
        "additionalPingWebhooks": webhooks,
        "fleetComms": fleet_comms,
        "fleetDoctrines": doctrines,
        "fleetFormupLocations": formup_locations,
        "site_url": get_site_url(),
        "timezones_installed": timezones_installed(),
        "mainCharacter": request.user.profile.main_character,
        "useSlack": AA_FLEETPINGFORMATTER_USE_SLACK,
    }

    return render(request, "discordpingformatter/index.html", context)



# -*- coding: utf-8 -*-

"""
a couple of variable to use throughout the app
"""

default_app_config = "discordpingformatter.apps.AaDiscordPingFormatterConfig"

__version__ = "2.0.0a4"
__title__ = "Fleet Ping Formatter"



# Generated by Django 2.2.15 on 2020-09-12 15:06

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("discordpingformatter", "0002_initialize_db"),
    ]

    operations = [
        migrations.AlterField(
            model_name="discordpingtargets",
            name="discord_id",
            field=models.CharField(
                blank=True,
                help_text="ID of the Discord role to ping",
                max_length=255,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="discordpingtargets",
            name="name",
            field=models.OneToOneField(
                help_text="Name of the Discord role to ping",
                on_delete=django.db.models.deletion.CASCADE,
                to="auth.Group",
            ),
        ),
        migrations.AlterField(
            model_name="discordpingtargets",
            name="restricted_to_group",
            field=models.ManyToManyField(
                help_text="Restrict Discord group to the following group(s) ...",
                related_name="discord_role_require_groups",
                to="auth.Group",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="restricted_to_group",
            field=models.ManyToManyField(
                help_text="Restrict this webhook to the following group(s) ...",
                related_name="webhook_require_groups",
                to="auth.Group",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="url",
            field=models.CharField(
                help_text="URL of this webhook, e.g. https://discordapp.com/api/webhooks/123456/abcdef or https://hooks.slack.com/services/xxxx/xxxx",
                max_length=255,
                unique=True,
            ),
        ),
    ]
